<?php

/**
 * Plugin Name:       MIQID-Elementor
 * Description:       MIQID-Elementor extend Elementor with MIQID data.
 * Version:           1.3
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            MIQ ApS
 * Author URI:        https://miqid.com/
 * License:           GPL v3 or later
 * License URI:       http://www.gnu.org/licenses/gpl-3.0.txt
 */

namespace MIQID\Elementor;

use Elementor\Plugin;
use MIQID\Elementor\Widget\{ Data_Window, Dynamic_Images, Dynamic_Text, FormSave, IconList, Input, Text_Hide_If};
use MIQID\Elementor\Handler\{FormSave as hFormSave};

if ( ! defined( 'WPINC' ) ) {
	die();
}

class Elementor {
	const VERSION                   = "1.3";
	const MINIMUM_ELEMENTOR_VERSION = "2.0.0";
	const MINIMUM_PHP_VERSION       = "7.2";

	private static $_instance;

	public static function Instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	public function __construct() {
		spl_autoload_register( function ( $class ) {
			if ( stripos( $class, __NAMESPACE__ ) === 0 ) {
				$include_file = implode( DIRECTORY_SEPARATOR, [
					__DIR__,
					mb_strtolower( sprintf( '%s.php', strtr( $class, [ sprintf( '%s\\', __NAMESPACE__ ) => '', '\\' => DIRECTORY_SEPARATOR, ] ) ) ),
				] );

				/** @noinspection PhpIncludeInspection */
				@include $include_file;
			}
		} );

		add_action( 'plugins_loaded', [ $this, 'init' ] );
	}

	function init() {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';

		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			$this->_deactivate_plugin( [ $this, '_notice_php_version' ] );
		} else if ( ! is_plugin_active( 'miqid-core/miqid-core.php' ) ) {
			$this->_deactivate_plugin( [ $this, '_notice_missing_core' ] );
		} else if ( ! is_plugin_active( 'elementor/elementor.php' ) ) {
			$this->_deactivate_plugin( [ $this, '_notice_missing_elementor' ] );
		} else if ( ! did_action( 'elementor/loaded' ) ) {
			$this->_deactivate_plugin( [ $this, '_notice_missing_elementor' ] );
		} else if ( version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '<' ) ) {
			$this->_deactivate_plugin( [ $this, '_notice_elementor_version' ] );
		} else {
			$this->_table_visitor_counter();

			add_action( 'elementor/elements/categories_registered', [ $this, 'miqid_elementor_categories' ] );

			add_action( 'elementor/widgets/widgets_registered', [ $this, 'register_widgets' ] );

			hFormSave::Instance();
		}
	}

	function miqid_elementor_categories( $elements_manager ) {

		$elements_manager->add_category( 'miqid', [
			'title' => "MIQID",
			'icon'  => 'fa fa-plug',
		] );
	}

	public function register_widgets() {

		Plugin::instance()->widgets_manager->register_widget_type( new Data_Window() );
		Plugin::instance()->widgets_manager->register_widget_type( new Dynamic_Images() );
		Plugin::instance()->widgets_manager->register_widget_type( new Dynamic_Text() );
		Plugin::instance()->widgets_manager->register_widget_type( new FormSave() );
		Plugin::instance()->widgets_manager->register_widget_type( new IconList() );
		Plugin::instance()->widgets_manager->register_widget_type( new Input() );
		Plugin::instance()->widgets_manager->register_widget_type( new Text_Hide_If() );

	}

	function _table_visitor_counter() {
		global $wpdb;
		$dbVersion = get_option( '_miqid_table_visitor_counter', "1.0.0" );

		if ( version_compare( $dbVersion, self::VERSION, '<' ) ) {

			$query = "CREATE TABLE {$wpdb->prefix}visitor_counter (
    ID INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    Store VARCHAR(50) NOT NULL,
    Department VARCHAR(50) NOT NULL,
    VisitorDate DATE NOT NULL,
    User VARCHAR(50) NOT NULL DEFAULT '',
    State ENUM ('CheckIn', 'CheckOut') NOT NULL DEFAULT 'CheckOut',
    CreatedDatetime DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
    ModifiedDate DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
    UNIQUE KEY store_visitor (Store,Department,VisitorDate,User)
) {$wpdb->get_charset_collate()}";

			require_once ABSPATH . '/wp-admin/includes/upgrade.php';
			dbDelta( $query );

			update_option( '_miqid_table_visitor_counter', self::VERSION );
		}
	}

	/**
	 * @param callable $cb
	 */
	public function _deactivate_plugin( callable $cb ) {
		deactivate_plugins( 'miqid-elementor/miqid-elementor.php' );
		add_action( 'admin_notices', $cb );
	}

	public function _notice_missing_elementor() {
		?>
		<div class="notice notice-error is-dismissible">
			<p><?php _e( 'MIQID-Elementor has been deactivated, missing MIQID-elementor', 'miqid-woo' ) ?></p>
		</div>
		<?php
	}

	public function _notice_missing_core() {
		?>
		<div class="notice notice-error is-dismissible">
			<p><?php _e( 'MIQID-Elementor has been deactivated, missing MIQID-Core', 'miqid-woo' ) ?></p>
		</div>
		<?php
	}

	public function _notice_php_version() {
		?>
		<div class="notice notice-error is-dismissible">
			<p><?= sprintf( __( 'MIQID-Elementor has been deactivated, wrong PHP Version: %s', 'miqid-woo' ), PHP_VERSION ) ?></p>
		</div>
		<?php
	}

	public function _notice_elementor_version() {
		?>
		<div class="notice notice-error is-dismissible">
			<p><?= sprintf( __( 'MIQID-Elementor has been deactivated, wrong PHP Version: %s', 'miqid-woo' ), ELEMENTOR_VERSION ) ?></p>
		</div>
		<?php
	}
}

Elementor::Instance();